import Data.Time
import Data.Maybe
import Control.Monad
import Control.Monad.IO.Class
import Control.DeepSeq
import Data.ByteString.Char8 hiding (putStrLn)
import Network.MessagePackRpc.Client

rpcRead :: ByteString -> Client (Maybe ByteString)
rpcRead = call "read"

rpcWrite :: ByteString -> ByteString -> Client ()
rpcWrite = call "write"

writeLog :: String -> Client ()
writeLog msg = liftIO $ do
  t <- getCurrentTime
  putStrLn $ "[" ++ show t ++ "] " ++ msg

actionsNumber :: Int
actionsNumber = 100000

main = runClient (pack "localhost") 1234 $ do
  writeLog "Preparing kvList..." 
  let kvList = force [ pack $ show x | x <- [1..actionsNumber] ]
  writeLog "Writing to cache..." 
  forM_ kvList $ \x -> rpcWrite x x
  writeLog "Writing finished" 
  writeLog "Reading from cache..."
  readRslt <- forM kvList rpcRead
  writeLog "Reading finished"
  let s = sum [ (read $ unpack x) :: Integer | x <- catMaybes readRslt]
  writeLog $ "Sum: " ++ show s
